%
% J. Bioucas Dias and G. Valadao (c)2009 
%
% This is a demo illustrating the absolute
% phase estimation performance of CAPE.
%
% See "CAPE: Combinatorial Absolute Phase Estimation"
% by G. Valadao and J. Bioucas-Dias, 2009.  
%
%              %%%%%%%%%%%%
%
% CAPEdemo2: Clipped Gaussian (quarter clipped)
%  
% observation model  z = exp(j phi) + n; n is Gaussian
%

% constants
I = sqrt(-1);

% noise 
%%% SIGMA %%%%
SIG = 0.1; %%%
%%% SIGMA %%%%
sigma = SIG/sqrt(2);
SNR = 10*log10(1/2/sigma^2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% GENERATE ORIGINAL PHASE, NOISE, AND OBSERVED DATA %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
M=100;
N=100;
varphi=gaussele(M,N,14*pi,10,15); 
varphi(1:50,1:50) = 0;

% generate noise
noise = sigma*(randn(M,N)+I*randn(M,N));
% generate observed data
z = exp(I*varphi)+ noise;
% compute the interferogram;
phi = angle(z);

% show interferogram
figure(1);
imagesc(phi); colormap(gray); axis off; 
title(strcat('Wrapped phase: SNR = ',num2str(SNR,3),' dB'), ...
     'FontName', 'TimesNewRoman','FontSize', 14); colorbar;
set(1,'Position',[100,300,550,420])

% CAPE 
figure(2); colormap(gray); 
set(2,'Position',[400,300,550,420])
 
[x,flowseries2] = qpbo_cos(phi,0.4,0.4,1,phi);

% compute the RMSE 
error = x - varphi;
RMSE = std(error(:))
title(strcat('Estimated absolute phase: RMSE = ',num2str(RMSE,2),' rad'), ...
     'FontName', 'TimesNewRoman','FontSize', 14); colorbar;axis off;

% show absolute phase estimation rewrapped
figure(3);colormap(gray);
imagesc(angle(exp(I*x))); colormap gray; axis off; 
title('Estimated absolute phase rewrapped','FontName', 'TimesNewRoman','FontSize', 14); colorbar;
set(3,'Position',[700,300,550,420])
figure(2);axis off;

