function [x,flowseries] = qpbo_cos(y,mu,p,bopt,x0)
%        [x,flowseries] = qpbo_cos(y,mu,p,bopt,x0)
% 
% J. Bioucas-Dias and G. Valadao (c)2009
% 
% Copyright acknowledgements: see the copyright notices inside QBMMGC_v0 
% (function invoked below)  
%
% --- Input parameters -----
%
% y  -> wrapepped phase -------
% mu -> smoothing parameter --- 
% p  -> potential parameter v(x) = min(x^2, th^(2-p)|x|^p)
% bopt ->  binary optimization 
%          1 - QPBMM
%          2 - QPBO
%          3 - QPBOI
%          4 - QPBOP
% x0 -> initial x0

global UE PI PE m n;

[m,n] = size(y);
ymax = max(y(:));
ymin = min(y(:));

% Initial resolution
delta = 2*pi*2;
preverg = inf;
counter = 0;
flowseries = [];


% Initial x
if nargin == 5
    x = x0;
else
 x = y;
end

cliques =[0 1; 1 0];
% clique [0 1]
nodes_c01 = p_clique_set([0 1],[m n]);
% clique [1 0]
nodes_c10 = p_clique_set([1 0],[m n]);


% set of cliques
PI = uint32([nodes_c01  nodes_c10]);
% initialize pairwise interactions

% options for QPBO

options = cell(2,1);

% scale 
c = 0;

% handle suplying a labeling to be improved
 W = @(x) iniLabeling(x);
 
% QPBO plus improve
options{1} = int32([m*n 2 0 0]);
options{2} = W ;

% QPBO plus Probe
optionsP{1} = int32([m*n 0 1 0]);


th=pi;

for depth = 1:8
    delta = delta/2
    if delta >= 2*pi
            successdown = true;
    else
            successdown = false;
            th=pi;
            preverg = inf;
    end
    successup = false; 

    th
    while ((~successup)|(~successdown))
        
        if ~successup
            % Unary terms
            E0 = -cos(y-x);      
            E1 = -cos(y-x-delta);
            
            UE = [E0(:)'; E1(:)'];
            
            clear A;
            
            % Pairwise terms for clique [0 1]
            aux =  mu*clique_energy(diff(x,1,2), p,th);         % E00
            A(1,:) =   aux(:)';                     
            aux =  mu*clique_energy(diff(x,1,2)+delta, p,th);   % E01
            A(2,:) = aux(:)';
            aux =  mu*clique_energy(diff(x,1,2)-delta, p,th);   % E10 
            A(3,:) =  aux(:)';
            A(4,:) =  A(1,:);                     % E11
            
            PE = A;
            clear A;
            
            % Pairwise terms for clique [1 0]
            aux =  mu*clique_energy(diff(x,1,1), p,th);         % E00
            A(1,:) =   aux(:)';                     
            aux =  mu*clique_energy(diff(x,1,1)+delta, p,th);   % E01
            A(2,:) = aux(:)';
            aux =  mu*clique_energy(diff(x,1,1)-delta, p,th);   % E10
            A(3,:) = aux(:)';
            A(4,:) =  A(1,:);                     % E11
            
            PE = [PE A];
            
                 super_modular = (PE(1,:)+PE(4,:)-PE(2,:)-PE(3,:));
                 mask = super_modular > 0;
                 sum(mask)
        
            switch bopt
                case 1
                    [seg, stat,prev_energy] = QBMMGC_v0(UE, PI, PE);
                    clc;
                case 2
                    [seg,stat] = QPBOPmex(UE,PI,PE);    
                case 3
                    [seg,stat] = QPBOPmex(UE,PI,PE,options);  
                case 4
                    [seg,stat] = QPBOPmex(UE,PI,PE,optionsP);
            end
              if (stat(1)+stat(3)) > 0
                  fprintf('\n number of unlabelled pixels -> %d',stat(1))
                  fprintf('\n number of unlabelled pixels after QPBOP -> %d',stat(3))
              end
% 
%             
            seg=reshape(double(seg),m,n);
            erg = mu*sum(sum(clique_energy(diff(x+delta*seg,1,1),p,th) )) + ...
                  mu*sum(sum(clique_energy(diff(x+delta*seg,1,2),p,th))) + ...
                       -sum(sum(cos(y-(x+delta*seg))));
            
            if  erg >= preverg

                successup = logical(1);
            else
                fprintf('up')
                x = x + delta*seg;
                preverg = erg
                flowseries = [flowseries; erg];
                imagesc(x); figure(gcf);
                drawnow;
            end
        end


        if ~successdown
            % Unary terms
             E0 = -cos(y-x)+c*x.^2;
             E1 = -cos(y-x+delta)+c*(x-delta).^2;
            
            UE = [E0(:)'; E1(:)'];
            
            clear A;
            
            % Pairwise terms for clique [0 1]
            aux =  mu*clique_energy(diff(x,1,2), p,th);         % E00
            A(1,:) =   aux(:)';                     
            aux =  mu*clique_energy(diff(x,1,2)-delta, p,th);   % E01
            A(2,:) = aux(:)';
            aux =  mu*clique_energy(diff(x,1,2)+delta, p,th);   % E10
            A(3,:) = aux(:)';
            A(4,:) =  A(1,:);                     % E11
                        
            
            PE = A;
            clear A;
            
            % Pairwise terms for clique [1 0]
            aux =  mu*clique_energy(diff(x,1,1), p,th);         % E00
            A(1,:) =   aux(:)';                     
            aux =  mu*clique_energy(diff(x,1,1)-delta, p,th);   % E01
            A(2,:) = aux(:)';
            aux =  mu*clique_energy(diff(x,1,1)+delta, p,th);   % E10
            A(3,:) = aux(:)';
            A(4,:) =  A(1,:);                     % E11
            
            PE = [PE A];
                       

                super_modular = (PE(1,:)+PE(4,:)-PE(2,:)-PE(3,:));
                mask = super_modular > 0;
                sum(mask)
%            
%                 
            switch bopt
                case 1
                    [seg, stat,prev_energy] = QBMMGC_v0(UE, PI, PE);
                    clc;
                case 2
                    [seg,stat] = QPBOPmex(UE,PI,PE);    
                case 3
                    [seg,stat] = QPBOPmex(UE,PI,PE,options);  
                case 4
                    [seg,stat] = QPBOPmex(UE,PI,PE,optionsP);
            end
              if (stat(1)+stat(3)) > 0
                  fprintf('\n number of unlabelled pixels -> %d',stat(1))
                  fprintf('\n number of unlabelled pixels after QPBOP -> %d',stat(3))
              end

            
             if (stat(1)+stat(3)) > 0
                  fprintf('\n number of unlabelled pixels -> %d',stat(1))
                  fprintf('\n number of unlabelled pixels after QPBOP -> %d',stat(3))
              end

         
            seg=reshape(double(seg),m,n);
            erg = mu*sum(sum(clique_energy(diff(x-delta*seg,1,1),p,th) )) + ...
                  mu*sum(sum(clique_energy(diff(x-delta*seg,1,2),p,th))) + ...
                       -sum(sum(cos(y-(x-delta*seg))));
            
            if erg >= preverg
                successdown = logical(1);
            else
                fprintf('down')
                x = x - delta*seg;
                preverg = erg
                flowseries = [flowseries; erg];
                imagesc(x); figure(gcf);
                drawnow;
            end
        end


    end
end


function seg = iniLabeling(x)
global UE PI PE m n;

x=double(x);
%set unlabeled to zero
x=x.*(x>=0);

seg = logical(QBMMGC_v0(UE, PI, PE));

return;

